@@ -1,4 +1,4 @@
-package CTK::DBI; # $Id: DBI.pm 250 2019-05-09 12:09:57Z minus $
+package CTK::DBI; # $Id: DBI.pm 272 2019-09-26 08:45:46Z minus $
 use strict;
 use utf8;
 
@@ -10,7 +10,7 @@ CTK::DBI - Database independent interface for CTKlib
 
 =head1 VERSION
 
-Version 2.29
+Version 2.30
 
 =head1 SYNOPSIS
 
@@ -27,10 +27,13 @@ Version 2.29
             -connect_to => 5,
             -request_to => 60
             #-attr      => {},
+            #-prepare_attr => {},
             #-debug     => 1,
         );
 
-    my $dbh = $mso->connect;
+    my $dbh = $mso->connect or die($mso->error);
+
+    die($mso->error) if $mso->error;
 
     # Table select (as array)
     my @result = $mso->table($sql, @inargs);
@@ -44,7 +47,7 @@ Version 2.29
     # Record (as hash)
     my %result = $mso->recordh($sql, @inargs);
 
-    # Fields (as scalar)
+    # Field (as scalar)
     my $result = $mso->field($sql, @inargs);
 
     # SQL
@@ -66,11 +69,18 @@ For example: print($mso->field("select sysdate() from dual"));
             -connect_to => 5,
             -request_to => 60
             #-attr      => {},
+            #-prepare_attr => {},
             #-debug     => 1,
         );
 
 Create the DBI object
 
+=head2 error
+
+    die $mso->error if $mso->error;
+
+Returns error string
+
 =head2 connect
 
     my $dbh = $mso->connect;
@@ -79,7 +89,7 @@ Get DBH (DB handler)
 
 =head2 disconnect
 
-    my $mso = $mso->disconnect;
+    my $rc = $mso->disconnect;
 
 Forced disconnecting. Please not use this method
 
@@ -123,6 +133,20 @@ Get (select) table from database as array or hash
 
 See C<Changes> file
 
+=head1 VARIABLES
+
+=over 4
+
+=item B<$CTK::DBI::CTK_DBI_DEBUG>
+
+Debug mode flag. Default: 0
+
+=item B<$CTK::DBI::CTK_DBI_ERROR>
+
+General error string
+
+=back
+
 =head1 DEPENDENCIES
 
 L<DBI>, L<Sys::SigAction>
@@ -166,8 +190,9 @@ use constant {
 };
 
 our $CTK_DBI_DEBUG = 0;
+our $CTK_DBI_ERROR = "";
 use vars qw/$VERSION/;
-$VERSION = '2.29';
+$VERSION = '2.30';
 
 my $LOAD_SigAction = 0;
 eval 'use Sys::SigAction';
@@ -179,7 +204,7 @@ if ($es) {
         sub set_sig_handler($$;$$) { 1 };
         1;
     ';
-    _error("Package Sys::SigAction don't installed! Please install this package") unless WIN;
+    _debug("Package Sys::SigAction don't installed! Please install this package") unless WIN;
 } else {
     $LOAD_SigAction = 1;
 }
@@ -194,82 +219,185 @@ sub new {
           ['PASSWORD','PASS'],
           ['TIMEOUT_CONNECT','CONNECT_TIMEOUT','CNT_TIMEOUT','TIMEOUT_CNT','TO_CONNECT','CONNECT_TO'],
           ['TIMEOUT_REQUEST','REQUEST_TIMEOUT','REQ_TIMEOUT','TIMEOUT_REQ','TO_REQUEST','REQUEST_TO'],
-          ['ATTRIBUTES','ATTR','ATTRHASH','PARAMS'],
+          ['ATTRIBUTES','ATTR','ATTRS','ATTRHASH','PARAMS'],
+          ['PREPARE_ATTRIBUTES','PREPARE_ATTR','PREPARE_ATTRS'],
           ['DEBUG'],
-        ],@_);
-    if ($in[6]) {
+        ], @_);
+    if ($in[7]) {
         $CTK_DBI_DEBUG = 1;
     }
 
     # General arguments
     my %args = (
             dsn         => $in[0] || '',
-            user        => $in[1] || '',
-            password    => $in[2] || '',
+            user        => $in[1] // '',
+            password    => $in[2] // '',
             connect_to  => $in[3] || TIMEOUT_CONNECT,
             request_to  => $in[4] || TIMEOUT_REQUEST,
             attr        => $in[5] || undef,
+            prepare_attr=> $in[6] || undef,
+            debug       => $in[7] // 0,
             dbh         => undef,
+            error       => "", # Ok
         );
 
     # Connect
-    $args{dbh} = DBI_CONNECT(@args{qw/dsn user password attr connect_to/});
-
-    # Ok
-    _debug("--- DBI CONNECT {".$args{dsn}."} ---");
-
+    my $_err = "";
+    $args{dbh} = DBI_CONNECT($args{dsn}, $args{user}, $args{password}, $args{attr}, $args{connect_to}, \$_err);
     my $self = bless {%args}, $class;
+    if ($args{dbh}) { # Ok
+        _debug(sprintf("--- DBI CONNECT {%s} ---", $args{dsn}));
+    } else {
+        $self->_set_error($_err);
+    }
+
     return $self;
 }
+
+sub _set_error {
+    my $self = shift;
+    my $merr = shift;
+    my $dbh = $self->{dbh};
+
+    if (defined($merr)) {
+        $self->{error} = $merr;
+    } elsif ($dbh && $dbh->can('errstr')) {
+        $self->{error} = $self->{dbh}->errstr // '';
+    } elsif (defined($DBI::errstr)) {
+        $self->{error} = $DBI::errstr;
+    } else {
+        $self->{error} = "";
+    }
+    if ($dbh && $dbh->{PrintError}) {
+        carp(sprintf("%s: %s", __PACKAGE__, $self->{error})) if length($self->{error});
+    }
+
+    return undef;
+}
+
+sub error {
+    my $self = shift;
+    return $self->{error} // "";
+}
 sub connect {
-    # Returns dbh
+    # Returns dbh object
     my $self = shift;
     return $self->{dbh};
 }
 sub disconnect {
-    # before DESTROY
     my $self = shift;
-    DBI_DISCONNECT ($self->{dbh}) if $self->{dbh};
-    # Ok
-    _debug("--- DBI DISCONNECT {".($self->{dsn} || '')."} ---");
+    return unless $self->{dbh};
+    my $rc = $self->{dbh}->disconnect;
+    _debug(sprintf("--- DBI DISCONNECT {%s} ---", $self->{dsn} || ''));
+    return $rc;
 }
 sub field {
     my $self = shift;
-    DBI_EXECUTE_FIELD($self->{dbh},$self->{request_to},@_)
+    my @result = $self->record(@_);
+    return shift @result;
 }
 sub record {
     my $self = shift;
-    DBI_EXECUTE_RECORD($self->{dbh},$self->{request_to},@_)
+    my $sth = $self->execute(@_);
+    return () unless $sth;
+    my @result = $sth->fetchrow_array;
+    $sth->finish;
+    return @result;
 }
 sub recordh {
     my $self = shift;
-    DBI_EXECUTE_RECORDH($self->{dbh},$self->{request_to},@_)
+    my $sth = $self->execute(@_);
+    return () unless $sth;
+    my $rslt = $sth->fetchrow_hashref;
+    $rslt = {} unless $rslt && ref($rslt) eq 'HASH';
+    my %result = %$rslt;
+    $sth->finish;
+    return %result;
 }
 sub table {
     my $self = shift;
-    DBI_EXECUTE_TABLE($self->{dbh},$self->{request_to},@_)
+    my $sth = $self->execute(@_);
+    return () unless $sth;
+    my $rslt = $sth->fetchall_arrayref;
+       $rslt = [] unless $rslt && ref($rslt) eq 'ARRAY';
+    my @result = @$rslt;
+    $sth->finish;
+    return @result;
 }
 sub tableh {
     my $self = shift;
     my $key_field = shift; # See keys (http://search.cpan.org/~timb/DBI-1.607/DBI.pm#fetchall_hashref)
-    DBI_EXECUTE_TABLEH($self->{dbh},$key_field,$self->{request_to},@_)
+    my $sth = $self->execute(@_);
+    return () unless $sth;
+    my $rslt = $sth->fetchall_hashref($key_field);
+       $rslt = {} unless $rslt && ref($rslt) eq 'HASH';
+    my %result = %$rslt;
+    $sth->finish;
+    return %result;
 }
 sub execute {
     my $self = shift;
-    DBI_EXECUTE($self->{dbh},$self->{request_to},@_)
+    my $sql = shift // '';
+    my @inargs = @_;
+    my $dbh = $self->{dbh} || return;
+    return $self->_set_error("No statement specified") unless length($sql);
+    $self->_set_error(""); # Flush errors
+
+    # Prepare
+    my $prepare_attr = $self->{prepare_attr};
+    my %attr = ($prepare_attr && ref($prepare_attr) eq 'HASH') ? %$prepare_attr : ();
+    my $sth_ex = keys(%attr) ? $dbh->prepare($sql, {%attr}) : $dbh->prepare($sql);
+    unless ($sth_ex) {
+        return $self->_set_error(sprintf("Can't prepare statement \"%s\": %s", $sql, $dbh->errstr // 'unknown error'));
+    }
+
+    # Execute
+    my $rto = $self->{request_to};
+    my $count_execute     = 1;     # TRUE
+    my $count_execute_msg = 'ok';  # TRUE
+    eval {
+        local $SIG{ALRM} = sub { die "execute timed out ($rto sec)" } unless $LOAD_SigAction;
+        my $h = Sys::SigAction::set_sig_handler( 'ALRM' ,sub { die "execute timed out ($rto sec)" ; } );
+        eval {
+            alarm($rto);
+            unless ($sth_ex->execute(@inargs)) {
+                $count_execute     = 0; # FALSE
+                $count_execute_msg = $dbh->errstr;  # FALSE
+            }
+            alarm(0);
+        };
+        alarm(0);
+        die $@ if $@;
+    };
+    if ( $@ ) {
+        $count_execute     = 0; # FALSE
+        $count_execute_msg = $@;
+    }
+    unless ($count_execute) {
+        my @repsrgs = @inargs;
+        my $argb = "";
+        $argb = sprintf(" with bind variables: %s", join(", ", map {defined($_) ? sprintf("\"%s\"", $_) : 'undef'} @repsrgs))
+            if exists($inargs[0]);
+        return $self->_set_error(sprintf("Can't execute statement \"%s\"%s: %s", $sql, $argb || '', $count_execute_msg // 'unknown error'));
+    }
+
+    return $sth_ex;
 }
+
 sub DESTROY {
     my $self = shift;
     $self->disconnect();
 }
 sub DBI_CONNECT {
     # Connect
-    # $dbh = DBI_CONNECT($dsn, $user, $password, $attr)
+    # $dbh = DBI_CONNECT($dsn, $user, $password, $attr, $to, $error)
     # IN:
     #   <DSN>      - DSN
     #   <USER>     - DB Username
     #   <PASSWORD> - DB Password
     #   <ATTR>     - Attributes DBD::* (hash-ref)
+    #   <TIMEOUT>  - Timeout value
+    #   <\ERROR>   - Reference to error scalar
     # OUT:
     #   $dbh - DataBase Handler Object
     #
@@ -278,14 +406,16 @@ sub DBI_CONNECT {
     my $db_password = shift // '';
     my $db_attr     = shift || {}; # E.g., {ORACLE_enable_utf8 => 1}
     my $db_tocnt    = shift || TIMEOUT_CONNECT;
+    my $rerr        = shift;
+       $rerr = \$CTK_DBI_ERROR unless $rerr && ref($rerr) eq 'SCALAR';
 
     my $dbh;
 
     my $count_connect     = 1;     # TRUE
-    my $count_connect_msg = 'OK';  # TRUE
+    my $count_connect_msg = 'ok';  # TRUE
     eval {
-        local $SIG{ALRM} = sub { die "Connecting timeout \"$db_dsn\"" } unless $LOAD_SigAction;
-        my $h = Sys::SigAction::set_sig_handler( 'ALRM', sub { die "Connecting timeout \"$db_dsn\"" } );
+        local $SIG{ALRM} = sub { die "connect timed out ($db_tocnt sec)" } unless $LOAD_SigAction;
+        my $h = Sys::SigAction::set_sig_handler( 'ALRM', sub { die "connect timed out ($db_tocnt sec)" } );
         eval {
             alarm($db_tocnt); #implement 2 second time out
             unless ($dbh = DBI->connect($db_dsn, "$db_user", "$db_password", $db_attr)) {
@@ -303,152 +433,13 @@ sub DBI_CONNECT {
         $count_connect_msg = $@;
     }
     unless ($count_connect) {
-        # Error :(
-        _error("[".__PACKAGE__.": Connecting error \"$db_dsn\"] $count_connect_msg");
+        $$rerr = sprintf("Can't connect to \"%s\", %s", $db_dsn, $count_connect_msg // 'unknown error');;
     }
 
     return $dbh;
 }
-sub DBI_DISCONNECT {
-    my $dbh = shift || return 0;
-    my $rc = $dbh->disconnect;
-    return $rc;
-}
-sub DBI_EXECUTE_FIELD {
-    # Get field
-    # $result = DBI_EXECUTE_FIELD($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $sql - SQL
-    #   [@inargs] - For binding
-    # OUT:
-    #   $result - First [0] value
 
-    my @result = DBI_EXECUTE_RECORD(@_);
-    return $result[0] || '';
-}
-sub DBI_EXECUTE_RECORD {
-    # Returns one record (array)
-    # @result = DBI_EXECUTE_RECORD($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $sql - SQL
-    #   [@inargs] - Binding arguments
-    # OUT:
-    #   @result - Array of data
-    my $sth = DBI_EXECUTE(@_);
-    return undef unless $sth;
-    my @result = $sth->fetchrow_array;
-    $sth->finish;
-    return @result;
-}
-sub DBI_EXECUTE_RECORDH {
-    # Returns one record (hash)
-    # %result = DBI_EXECUTE_RECORDH($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $sql - SQL
-    #   [@inargs] - Binding arguments
-    # OUT:
-    #   %result - Hash of data
-    my $sth = DBI_EXECUTE(@_);
-    return undef unless $sth;
-    my %result = %{$sth->fetchrow_hashref || {}};
-    $sth->finish;
-    return %result;
-}
-sub DBI_EXECUTE_TABLE {
-    # Get all data (table)
-    # @result = DBI_EXECUTE_TABLE($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $sql - SQL
-    #   [@inargs] - Binding arguments
-    # OUT:
-    #   @result - Array
-
-    my $sth = DBI_EXECUTE(@_);
-    return undef unless $sth;
-    my @result = @{$sth->fetchall_arrayref};
-    $sth->finish;
-    return @result;
-}
-sub DBI_EXECUTE_TABLEH {
-    # Get all data (table)
-    # %result = DBI_EXECUTE_TABLEH($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $key_field - Key (http://search.cpan.org/~timb/DBI-1.607/DBI.pm#fetchall_hashref)
-    #   $sql - SQL
-    #   [@inargs] - Binding arguments
-    # OUT:
-    #   Rresult - Hash of Hashes
-    my $dbh       = shift;
-    my $key_field = shift;
-
-    my $sth = DBI_EXECUTE($dbh,@_);
-    return undef unless $sth;
-    my %result = %{$sth->fetchall_hashref($key_field) || {}};
-    $sth->finish;
-    return %result;
-}
-sub DBI_EXECUTE {
-    # Execute
-    # $sth = DBI_EXECUTE($dbh, $sql, @inargs)
-    # IN:
-    #   $dbh - DataBase Handler Object
-    #   $tor - TimeOut of Request
-    #   $sql - SQL
-    #   [@inargs] - Binding arguments
-    # OUT:
-    #   $sth_ex - statement header
-
-    my $dbh = shift || return 0;
-    my $tor = shift || TIMEOUT_REQUEST;
-    my $sql = shift || return 0;
-
-    my @inargs = ();
-    @inargs = @_ if exists $_[0];
-    my $argb = "";
-    my @repsrgs = @inargs;
-    $argb = sprintf("Params: %s", join(", ", map {defined($_) ? sprintf("\"%s\"",$_) : 'undef'} @repsrgs)) if exists $inargs[0];
-
-    my $sth_ex = $dbh->prepare($sql);
-    unless ($sth_ex) {
-        _error("[".__PACKAGE__.": Preparing error: $sql"."] ".$dbh->errstr);
-        return undef;
-    }
-
-    my $count_execute     = 1;     # TRUE
-    my $count_execute_msg = 'OK';  # TRUE
-    eval {
-        local $SIG{ALRM} = sub { die "Executing timeout" } unless $LOAD_SigAction;
-        my $h = Sys::SigAction::set_sig_handler( 'ALRM' ,sub { die "Executing timeout" ; } );
-        eval {
-            alarm($tor);
-            unless ($sth_ex->execute(@inargs)) {
-                $count_execute     = 0; # FALSE
-                $count_execute_msg = $dbh->errstr;  # FALSE
-            }
-            alarm(0);
-        };
-        alarm(0);
-        die $@ if $@;
-    };
-    if ( $@ ) {
-        $count_execute     = 0; # FALSE
-        $count_execute_msg = $@;
-    }
-    unless ($count_execute) {
-        # Error
-        _error("[".__PACKAGE__.": Executing error: $sql".($argb?" / $argb":'')."] $count_execute_msg");
-        return undef;
-    }
-
-    return $sth_ex || undef;
-}
 sub _debug { $CTK_DBI_DEBUG ? carp(@_) : 1 }
-sub _error { carp(@_) }
 
 1;
 
